package net.wanji.opt.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.gts.Tools;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.LocalDateTimeUtil;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.opt.common.CommonUtils;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.constant.CrossEnum;
import net.wanji.opt.constant.DirEnum;
import net.wanji.opt.controllerv2.report.vo.LaneDirNumVO;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.StrategyCrossResultMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.dao.mapper.evaluation.EventInfoMapper;
import net.wanji.opt.dao.mapper.judgeanalysis.AiOptimizeStatisticMapper;
import net.wanji.opt.dao.mapper.report.AnalysisGreenWaveOptimizeWeekMapper;
import net.wanji.opt.dao.mapper.report.AnalysisReportCrossOptimizeDetailMapper;
import net.wanji.opt.dto.report.GreenHistIndexDTO;
import net.wanji.opt.dto.report.StrategyGreenOptHistDTO;
import net.wanji.opt.entity.evaluation.EventInfo;
import net.wanji.opt.entity.judgeanalysis.AiOptimizeStatistic;
import net.wanji.opt.entity.report.AnalysisGreenWaveOptimizeWeek;
import net.wanji.opt.entity.report.AnalysisReportCrossOptimizeDetail;
import net.wanji.opt.po.base.CrossLaneDataHistPoExtend;
import net.wanji.opt.po.base.EventStatisticPo;
import net.wanji.opt.service.GreenwaveHistProvider;
import net.wanji.opt.synthesis.pojo.StrategyCrossResultEntity;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.DateTimeUtils;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author fengyi
 * @date 2025/4/9
 * @description
 */
@Component
@Configurable
@EnableScheduling
@Slf4j
@Profile("!dev")
public class ReportGreenAndCrossOptimizeStatisticTask {

    @Resource
    StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    StrategyCrossResultMapper strategyCrossResultMapper;
    @Resource
    GreenwaveHistProvider greenwaveHistProvider;
    @Resource
    GreenwaveHistoryMapper greenwaveHistoryMapper;
    @Resource
    EventInfoMapper eventInfoMapper;
    @Resource
    AnalysisGreenWaveOptimizeWeekMapper analysisGreenWaveOptimizeWeekMapper;
    @Resource
    AnalysisReportCrossOptimizeDetailMapper analysisReportCrossOptimizeDetailMapper;
    @Resource
    HoloEventMapper holoEventMapper;
    @Resource
    AiOptimizeStatisticMapper aiOptimizeStatisticMapper;


    /**
     * 干线策略优化时长计算
     */
    @Scheduled(cron = "0 40 2 ? * 1")
    public void optimizeGreenWaveDurationCal() {
        long st = System.currentTimeMillis();
        try {

            List<AnalysisGreenWaveOptimizeWeek> durationList = new ArrayList<>();

            List<StrategyGreenOptHistDTO> list = strategyGreenOptHistMapper.findGreenWaveOptHist();
            Map<Integer, List<StrategyGreenOptHistDTO>> groupByGreenId = list.stream().collect(Collectors.groupingBy(o -> o.getGreenId()));

            for (Map.Entry<Integer, List<StrategyGreenOptHistDTO>> entry : groupByGreenId.entrySet()) {

                List<StrategyGreenOptHistDTO> value = entry.getValue();
                value = value.stream().sorted(Comparator.comparing(StrategyGreenOptHistDTO::getControlTime)).collect(Collectors.toList());
                int i = 0;
                for (StrategyGreenOptHistDTO result : value) {
                    if (result.getControlMethod() == -1) {
                        i++;
                        continue;
                    }
                    int seconds = 900;
                    if (i != value.size() - 1) {
                        if (value.get(i + 1).getControlMethod() == -1) {
                            //下一条记录为取消状态，当前取消时间-上次下方时间
                            seconds = Seconds.secondsBetween(new DateTime(result.getControlTime()), new DateTime(value.get(i + 1).getControlTime())).getSeconds();
                            if (seconds > result.getControlDuration()) {
                                seconds = result.getControlDuration();
                            }
                        } else if (result.getControlMethod() != -1) {
                            //两条记录之前没有取消状态，取ControlDuration
                            seconds = result.getControlDuration();
                        }
                    } else if (result.getControlMethod() != -1) {
                        //最后一天记录不为-1取消状态
                        seconds = Seconds.secondsBetween(new DateTime(result.getControlTime()), new DateTime(new Date())).getSeconds();
                        // seconds = Math.abs(seconds);
                        if (seconds > result.getControlDuration()) {
                            seconds = result.getControlDuration();
                        }

                    }

                    result.setDuration(seconds);
                    //优化控制时段-开始时间
                    String controlStartTime = DateUtil.formatDate(result.getControlTime(), "yyyy-MM-dd HH:mm:ss");
                    //优化控制时段-截止时间
                    String controlEndTime = new DateTime(result.getControlTime()).plusSeconds(seconds).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));
                    GreenHistIndexDTO dto = greenwaveHistProvider.findGreenIndexByDateScope(result.getGreenId(), controlStartTime, controlEndTime);

                    //上周策略同期-开始时间
                    DateTime lastWeekDateTime = new DateTime(result.getControlTime()).minusWeeks(1);
                    String lastWeekStartTime = DateUtil.formatDate(lastWeekDateTime.toDate(), "yyyy-MM-dd HH:mm:ss");
                    //上周策略同期-截止时间
                    String lastWeekEndTime = lastWeekDateTime.plusSeconds(seconds).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));
                    GreenHistIndexDTO lastWeekDto = greenwaveHistProvider.findGreenIndexByDateScope(result.getGreenId(), lastWeekStartTime, lastWeekEndTime);

                    AnalysisGreenWaveOptimizeWeek data = new AnalysisGreenWaveOptimizeWeek();
                    data.setGreenId(result.getGreenId());
                    data.setStragetyId(result.getStragetyId());
                    data.setRoadDirection(result.getRoadDirection());
                    if (dto != null) {
                        data.setCapacity(dto.getCapacity());
                        data.setDelayTime(dto.getDelayTime());
                        data.setSpeed(dto.getSpeed());
                        data.setTravelTime(dto.getTravelTime());
                        data.setStopTimes(dto.getStopTimes());
                        data.setCongestIndex(dto.getTrafficIndex());
                    }
                    if (lastWeekDto != null) {
                        data.setLastWeekCapacity(lastWeekDto.getCapacity());
                        data.setLastWeekDelayTime(lastWeekDto.getDelayTime());
                        data.setLastWeekSpeed(lastWeekDto.getSpeed());
                        data.setLastWeekTravelTime(lastWeekDto.getTravelTime());
                        data.setLastWeekStopTimes(lastWeekDto.getStopTimes());
                        data.setLastWeekCongestIndex(lastWeekDto.getTrafficIndex());
                    }
                    data.setWeekDay(result.getWeekDay());
                    data.setOptimizeStartTime(LocalDateTime.parse(controlStartTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                    data.setOptimizeEndTime(LocalDateTime.parse(controlEndTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                    data.setOptimizeCount(1);
                    data.setOptimizeDuration(seconds);
                    data.setWeekStartTime(LocalDateTimeUtil.convertDateToLDT(result.getWeekStartDay()));
                    data.setWeekEndTime(LocalDateTimeUtil.convertDateToLDT(result.getWeekEndDay()));
                    data.setYearWeek(result.getYearWeek());
                    data.setInsertTime(LocalDateTime.now());
                    durationList.add(data);
                    i++;
                }
            }
            if (!durationList.isEmpty()) {
                analysisGreenWaveOptimizeWeekMapper.insertBatch(durationList);
            }
            long et = System.currentTimeMillis();
            log.info("批量保存干线优化数据据耗时：{}ms，size:{}", et - st, durationList.size());

        } catch (Exception e) {
            log.error("", e);
        }
    }


    /**
     * 单点策略优化时长计算
     */
//    @Scheduled(cron = "0 0/6 *  * * * ")
    @Scheduled(cron = "0 20 3  * * * ")
    public void optimizeDurationCal() {

        long st = System.currentTimeMillis();
        try {

            //路口策略下方记录
            Date nowDate = cn.hutool.core.date.DateUtil.date();
            Date beginOfDay = cn.hutool.core.date.DateUtil.beginOfDay(nowDate);//.lastWeek();//

            beginOfDay = cn.hutool.core.date.DateUtil.lastWeek();
            beginOfDay = cn.hutool.core.date.DateUtil.beginOfWeek(beginOfDay);

            Date endOfDay = cn.hutool.core.date.DateUtil.endOfDay(nowDate);//DateUtil.endOfDay(DateUtil.date());

            while (beginOfDay.before(endOfDay)) {

                String startDate = cn.hutool.core.date.DateUtil.format(beginOfDay, "yyyy-MM-dd HH:mm:ss");
                beginOfDay = cn.hutool.core.date.DateUtil.offsetDay(beginOfDay, 1);
                String endDate = cn.hutool.core.date.DateUtil.format(beginOfDay, "yyyy-MM-dd HH:mm:ss");

                //==========================================================================================================//
                long st1 = System.currentTimeMillis();
                //查询策略优化记录
                QueryWrapper<StrategyCrossResultEntity> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("response_code", 200);
                queryWrapper.between("issue_time", startDate, endDate);
                List<StrategyCrossResultEntity> list = strategyCrossResultMapper.selectList(queryWrapper);
                for (StrategyCrossResultEntity entity : list) {
                    if (entity.getCurrentAlgo() == 2) {
                        int seconds = 900;
                        entity.setDuration(seconds);
                    } else if (entity.getCurrentAlgo() == 3) {
                        Integer rtnType = entity.getRtnType();
                        if (Objects.equals(1, rtnType)) {
                            //切换相位，持续时长为当前相位剩余时长，固定为1秒【新算法】
                            entity.setDuration(1);
                        } else if (Objects.equals(2, rtnType)) {
                            //延长相位，持续时长为当前相位剩余时长
                            entity.setDuration(entity.getExtendTime());
                        }
                    } else if (entity.getCurrentAlgo() == 1) {
                        //相位切换，持续时长给固定1秒
                        if (CrossEnum.isSwitchPhase(CrossEnum.getTypeEnum(entity.getRtnType().toString()))) {
                            entity.setDuration(1);
                        }
                        if (CrossEnum.isExtendPhase(CrossEnum.getTypeEnum(entity.getRtnType().toString()))) {
                            entity.setDuration(entity.getExtendTime());
                        }
                    }
                }
                long et1 = System.currentTimeMillis();
                log.info("查询策略优化记录耗时：{}ms，参数：{}", et1 - st1, queryWrapper.toString());
                //==========================================================================================================//
                List<AnalysisReportCrossOptimizeDetail> optimizeList = new ArrayList<>();

                Map<String, List<StrategyCrossResultEntity>> groupByCross = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getCurrentAlgo()));
                for (Map.Entry<String, List<StrategyCrossResultEntity>> entry : groupByCross.entrySet()) {
                    String[] sps = entry.getKey().split("_");
                    String crossId = sps[0];
                    String strategy = sps[1];
                    String eventType = null;
                    if (Objects.equals("1", strategy)) {
                        eventType = "701";
                    } else if (Objects.equals("2", strategy)) {
                        eventType = "702";
                    } else if (Objects.equals("3", strategy)) {
                        eventType = "703";
                    } else if (Objects.equals("4", strategy)) {
                        eventType = "704";
                    }

                    List<StrategyCrossResultEntity> value = entry.getValue();

                    StrategyCrossResultEntity minTime = value.stream().min(Comparator.comparing(o -> o.getIssueTime())).orElse(new StrategyCrossResultEntity());
                    StrategyCrossResultEntity maxTime = value.stream().max(Comparator.comparing(o -> o.getIssueTime())).orElse(new StrategyCrossResultEntity());
                    int duration = value.stream().mapToInt(o -> o.getDuration()).sum();
                    long count = value.size();

                    //优化控制时段-开始时间
                    String controlStartTime = DateUtil.formatDate(minTime.getIssueTime(), "yyyy-MM-dd HH:mm:ss");
                    //优化控制时段-截止时间
                    String controlEndTime = new DateTime(maxTime.getIssueTime()).plusSeconds(duration).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));

                    DateTime optStartTime = DateTime.parse(controlStartTime, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
                    String optStartTimeStr = controlStartTime.substring(0, controlStartTime.indexOf(" ") + 1) + Tools.fillZeroFront(optStartTime.getHourOfDay() + "", 2) + ":" + Tools.fillZeroFront((int) Math.ceil(optStartTime.getMinuteOfHour() / 5) * 5 + "", 2) + ":00";
                    //===========================================================================//
                    Map<String, Object> params = new HashMap<>();
                    params.put("crossId", crossId);
                    params.put("startDate", optStartTimeStr);
                    params.put("endDate", controlEndTime);
                    params.put("groupType", "0");
                    params.put("objectType", "4");
                    //本周指标数据
                    List<CrossLaneDataHistPoExtend> thisWeekList = greenwaveHistoryMapper.findCrossObjectIndex(params);
                    AnalysisReportCrossOptimizeDetail thisData = this.getWeedData(thisWeekList);
                    //===========================================================================//
                    //上周策略同期-开始时间
                    DateTime lastWeekDateStartTime = optStartTime.minusWeeks(1);
                    DateTime lastWeekDateEndTime = new DateTime(maxTime.getIssueTime()).plusSeconds(duration).minusWeeks(1);
                    String lastWeekStartTime = lastWeekDateStartTime.toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));

                    String lastOptStartTimeStr = lastWeekStartTime.substring(0, lastWeekStartTime.indexOf(" ") + 1) + Tools.fillZeroFront(lastWeekDateStartTime.getHourOfDay() + "", 2) + ":" + Tools.fillZeroFront((int) Math.ceil(lastWeekDateStartTime.getMinuteOfHour() / 5) * 5 + "", 2) + ":00";
                    String lastWeekEndTime = lastWeekDateEndTime.toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));

                    params.put("startDate", lastOptStartTimeStr);
                    params.put("endDate", lastWeekEndTime);
                    //上周指标数据
                    List<CrossLaneDataHistPoExtend> lastWeekList = greenwaveHistoryMapper.findCrossObjectIndex(params);
                    AnalysisReportCrossOptimizeDetail lastData = this.getWeedData(lastWeekList);

                    DateTime issueTime = new DateTime(minTime.getIssueTime());
                    //===========================================================================//
                    AnalysisReportCrossOptimizeDetail detail = new AnalysisReportCrossOptimizeDetail();
                    detail.setCrossId(minTime.getCrossId());
                    detail.setYearWeek(Integer.parseInt(issueTime.getYear() + "" + issueTime.getWeekOfWeekyear()));
                    detail.setWeekDay(issueTime.getDayOfWeek() + 1);
                    detail.setWeekStartTime(cn.hutool.core.date.DateUtil.beginOfWeek(minTime.getIssueTime()).toLocalDateTime());

                    detail.setWeekEndTime(cn.hutool.core.date.DateUtil.endOfWeek(minTime.getIssueTime()).toLocalDateTime());
                    detail.setOptimizeCount((int) count);
                    detail.setOptimizeDuration(duration);
                    detail.setOptimizeStartTime(LocalDateTime.parse(controlStartTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                    detail.setOptimizeEndTime(LocalDateTime.parse(controlEndTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                    detail.setStragetyId(minTime.getCurrentAlgo());
                    detail.setCapacity(thisData.getCapacity());
                    detail.setStopTimes(thisData.getStopTimes());
                    detail.setDelayTime(thisData.getDelayTime());
                    detail.setMaxQueueLength(thisData.getMaxQueueLength());
                    detail.setLastWeekCapacity(lastData.getCapacity());
                    detail.setLastWeekStopTimes(lastData.getStopTimes());
                    detail.setLastWeekDelayTime(lastData.getDelayTime());
                    detail.setLastWeekMaxQueueLength(lastData.getMaxQueueLength());
                    detail.setInsertTime(LocalDateTime.now());

                    //===========================================================================//
                    String eventSerialNumber = minTime.getEventId();
                    Map<String, Object> eventParma = new HashMap<>();
                    eventParma.put("eventSerialNumber", eventSerialNumber);
                    List<EventInfo> eventInfoList = eventInfoMapper.findByMap(eventParma);
                    //获取事件方向
                    if (!eventInfoList.isEmpty()) {
                        String dir = eventInfoList.get(0).getDir();
                        detail.setDirs(CommonUtils.getEventHappenDirName(dir));
                    }
                    //===========================================================================//
                    params.clear();
                    params.put("crossId", crossId);
                    params.put("dt", DateUtil.formatDate(minTime.getIssueTime(), "yyyyMMdd"));
                    params.put("eventType", eventType);
                    EventStatisticPo countPo = holoEventMapper.findCrossEventCount(params);
                    if (countPo != null) {
                        detail.setProblemCount(countPo.getEventCount());
                    }
                    params.put("dt", DateUtil.formatDate(lastWeekDateStartTime.toDate(), "yyyyMMdd"));
                    //===========================================================================//
                    EventStatisticPo lastCountPo = holoEventMapper.findCrossEventCount(params);
                    if (lastCountPo != null) {
                        detail.setLastWeekOptimizeCount(lastCountPo.getEventCount());
                    }
                    //===========================================================================//
                    AiOptimizeStatistic param = new AiOptimizeStatistic();
                    param.setCrossId(crossId);
                    param.setStatisticTime(Integer.valueOf( DateUtil.formatDate(lastWeekDateStartTime.toDate(), "yyyyMMdd")));
                    param.setStrategy(Integer.valueOf(strategy));
                    List<AiOptimizeStatistic> lastOptList = aiOptimizeStatisticMapper.findAiOptimizeStatisticByCondition(param);
                    if (!lastOptList.isEmpty()) {
                        detail.setLastWeekOptimizeDuration(lastOptList.get(0).getOptimizeDuration());
                        detail.setLastWeekOptimizeCount(lastOptList.get(0).getOptimizeCount());
                    }

                    //===========================================================================//
                    optimizeList.add(detail);

                    if (optimizeList.size() % 100 == 0) {
                        analysisReportCrossOptimizeDetailMapper.insertBatch(optimizeList);
                        optimizeList.clear();
                        log.info("计算路口周报-路口优分批入库耗时：{}ms，size:{} ,starDate:{}", System.currentTimeMillis() - et1, optimizeList.size(), startDate);
                    }
                }
                //==========================================================================================================//
                if (!optimizeList.isEmpty()) {
                    analysisReportCrossOptimizeDetailMapper.insertBatch(optimizeList);
                }
                long et = System.currentTimeMillis();
                log.info("计算路口周报-路口优化计算总耗时：{}ms，size:{}， starDate:{}", et - st, optimizeList.size(), startDate);


            }
        } catch (Exception e) {
            log.error("计算路口周报-路口优化计算出现异常", e);
        }


    }


    private AnalysisReportCrossOptimizeDetail getWeedData(List<CrossLaneDataHistPoExtend> list) {
        AnalysisReportCrossOptimizeDetail ret = new AnalysisReportCrossOptimizeDetail();

        if (!list.isEmpty()) {
            int capacity = 0;
            //通行能力计算
            if (list.size() > 3) {
                for (int i = 0; i < list.size(); i++) {
                    if (i + 2 < list.size()) {
                        int sum = list.get(i).getFlow() + list.get(i + 1).getFlow() + list.get(i + 2).getFlow();
                        if (sum * 4 > capacity) {
                            capacity = sum * 4;
                        }
                    }
                }
            } else {
                capacity = list.stream().mapToInt(CrossLaneDataHistPoExtend::getFlow).sum();
            }
            //最大停车次数
            double maxStopTimes = list.stream().map(CrossLaneDataHistPoExtend::getStopTimes).max(Comparator.comparing(Double::doubleValue)).orElse(0d);
            //最大延误时间
            double maxDelayTime = list.stream().map(CrossLaneDataHistPoExtend::getDelayTime).max(Comparator.comparing(Integer::intValue)).orElse(0);
            //最大排队
            int maxQueueLength = list.stream().map(CrossLaneDataHistPoExtend::getMaxQueueLength).max(Comparator.comparing(Integer::intValue)).orElse(0);

            ret.setCapacity(capacity);
            ret.setStopTimes(maxStopTimes);
            ret.setDelayTime(maxDelayTime);
            ret.setMaxQueueLength(maxQueueLength);
        }

        return ret;
    }


}
